@@ -26,6 +26,7 @@ import six
 import webob.exc
 
 import glance.api.policy
+from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import trust_auth
 from glance.common import utils
@@ -34,11 +35,13 @@ import glance.db
 import glance.gateway
 from glance.i18n import _, _LE, _LI
 import glance.notifier
+from glance.quota import keystone as ks_quota
 
 
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
+CONF.import_opt('public_endpoint', 'glance.api.versions')
 
 
 class ImageDataController(object):
@@ -102,24 +105,14 @@ class ImageDataController(object):
 
         self._restore(image_repo, image)
 
-    def _delete(self, image_repo, image):
-        """Delete the image.
-
-        :param image_repo: The instance of ImageRepo
-        :param image: The image that will be deleted
-        """
-        try:
-            if image_repo and image:
-                image.status = 'killed'
-                image_repo.save(image)
-        except Exception as e:
-            msg = (_LE("Unable to delete image %(image_id)s: %(e)s") %
-                   {'image_id': image.image_id,
-                    'e': encodeutils.exception_to_unicode(e)})
-            LOG.exception(msg)
-
     @utils.mutating
     def upload(self, req, image_id, data, size):
+        try:
+            ks_quota.enforce_image_size_total(req.context, req.context.owner)
+        except exception.LimitExceeded as e:
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=str(e),
+                                                      request=req)
+
         backend = None
         if CONF.enabled_backends:
             backend = req.headers.get('x-image-meta-store',
@@ -132,35 +125,42 @@ class ImageDataController(object):
                                                request=req,
                                                content_type='text/plain')
 
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(req.context,
+                                           authorization_layer=False)
         image = None
         refresher = None
         cxt = req.context
         try:
-            self.policy.enforce(cxt, 'upload_image', {})
             image = image_repo.get(image_id)
+            # NOTE(abhishekk): This is the right place to check whether user
+            # have permission to upload the image and remove the policy check
+            # later from the policy layer.
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                self.policy)
+            api_pol.upload_image()
             image.status = 'saving'
             try:
-                if CONF.data_api == 'glance.db.registry.api':
-                    # create a trust if backend is registry
-                    try:
-                        # request user plugin for current token
-                        user_plugin = req.environ.get('keystone.token_auth')
-                        roles = []
-                        # use roles from request environment because they
-                        # are not transformed to lower-case unlike cxt.roles
-                        for role_info in req.environ.get(
-                                'keystone.token_info')['token']['roles']:
-                            roles.append(role_info['name'])
-                        refresher = trust_auth.TokenRefresher(user_plugin,
-                                                              cxt.project_id,
-                                                              roles)
-                    except Exception as e:
-                        LOG.info(_LI("Unable to create trust: %s "
-                                     "Use the existing user token."),
-                                 encodeutils.exception_to_unicode(e))
+                # create a trust if backend is registry
+                try:
+                    # request user plugin for current token
+                    user_plugin = req.environ.get('keystone.token_auth')
+                    roles = []
+                    # use roles from request environment because they
+                    # are not transformed to lower-case unlike cxt.roles
+                    for role_info in req.environ.get(
+                            'keystone.token_info')['token']['roles']:
+                        roles.append(role_info['name'])
+                    refresher = trust_auth.TokenRefresher(user_plugin,
+                                                          cxt.project_id,
+                                                          roles)
+                except Exception as e:
+                    LOG.info(_LI("Unable to create trust: %s "
+                                 "Use the existing user token."),
+                             encodeutils.exception_to_unicode(e))
 
                 image_repo.save(image, from_state='queued')
+                ks_quota.enforce_image_count_uploading(req.context,
+                                                       req.context.owner)
                 image.set_data(data, size, backend=backend)
 
                 try:
@@ -169,7 +169,6 @@ class ImageDataController(object):
                     if refresher is not None:
                         # request a new token to update an image in database
                         cxt.auth_token = refresher.refresh_token()
-                        image_repo = self.gateway.get_repo(req.context)
                         image_repo.save(image, from_state='saving')
                     else:
                         raise
@@ -267,6 +266,12 @@ class ImageDataController(object):
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
+        except exception.LimitExceeded as e:
+            LOG.error(str(e))
+            self._restore(image_repo, image)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=str(e),
+                                                      request=req)
+
         except glance_store.StorageWriteDenied as e:
             msg = _("Insufficient permissions on image "
                     "storage media: %s") % encodeutils.exception_to_unicode(e)
@@ -280,7 +285,7 @@ class ImageDataController(object):
                    % {'id': image_id,
                       'e': encodeutils.exception_to_unicode(e)})
             LOG.error(msg)
-            self._delete(image_repo, image)
+            self._restore(image_repo, image)
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         except webob.exc.HTTPGone:
@@ -300,8 +305,31 @@ class ImageDataController(object):
 
     @utils.mutating
     def stage(self, req, image_id, data, size):
-        image_repo = self.gateway.get_repo(req.context)
-        image = None
+        try:
+            ks_quota.enforce_image_staging_total(req.context,
+                                                 req.context.owner)
+        except exception.LimitExceeded as e:
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=str(e),
+                                                      request=req)
+
+        image_repo = self.gateway.get_repo(
+            req.context, authorization_layer=False)
+        # NOTE(abhishekk): stage API call does not have its own policy but
+        # it requires get_image access, this is the right place to check
+        # whether user has access to image or not
+        try:
+            image = image_repo.get(image_id)
+        except exception.NotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.msg)
+
+        api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                            enforcer=self.policy)
+        try:
+            api_pol.modify_image()
+        except exception.Forbidden as e:
+            # NOTE(abhishekk): This will throw Forbidden if S-RBAC is not
+            # enabled
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
 
         # NOTE(jokke): this is horrible way to do it but as long as
         # glance_store is in a shape it is, the only way. Don't hold me
@@ -339,17 +367,27 @@ class ImageDataController(object):
             staging_store = _build_staging_store()
 
         try:
-            image = image_repo.get(image_id)
             image.status = 'uploading'
             image_repo.save(image, from_state='queued')
+            ks_quota.enforce_image_count_uploading(req.context,
+                                                   req.context.owner)
             try:
-                staging_store.add(
+                uri, size, id, store_info = staging_store.add(
                     image_id, utils.LimitingReader(
                         utils.CooperativeReader(data), CONF.image_size_cap), 0)
+                image.size = size
             except glance_store.Duplicate:
                 msg = _("The image %s has data on staging") % image_id
                 raise webob.exc.HTTPConflict(explanation=msg)
 
+            # NOTE(danms): Record this worker's
+            # worker_self_reference_url in the image metadata so we
+            # know who has the staging data.
+            self_url = CONF.worker_self_reference_url or CONF.public_endpoint
+            if self_url:
+                image.extra_properties['os_glance_stage_host'] = self_url
+            image_repo.save(image, from_state='uploading')
+
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
 
@@ -377,6 +415,12 @@ class ImageDataController(object):
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
+        except exception.LimitExceeded as e:
+            LOG.debug(str(e))
+            self._unstage(image_repo, image, staging_store)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=str(e),
+                                                      request=req)
+
         except glance_store.StorageWriteDenied as e:
             msg = _("Insufficient permissions on image "
                     "storage media: %s") % encodeutils.exception_to_unicode(e)
@@ -397,13 +441,19 @@ class ImageDataController(object):
                 self._restore(image_repo, image)
 
     def download(self, req, image_id):
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(
+            req.context, authorization_layer=False)
         try:
             image = image_repo.get(image_id)
             if image.status == 'deactivated' and not req.context.is_admin:
                 msg = _('The requested image has been deactivated. '
                         'Image data download is forbidden.')
                 raise exception.Forbidden(message=msg)
+            # NOTE(abhishekk): This is the right place to verify whether
+            # user has permission to download the image or not.
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                self.policy)
+            api_pol.download_image()
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
