@@ -11,8 +11,6 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import sys
-
 from oslo_config import cfg
 from oslo_log import log as logging
 from six.moves import configparser
@@ -82,11 +80,7 @@ Related options:
 """)),
 ]
 
-# SafeConfigParser was deprecated in Python 3.2
-if sys.version_info >= (3, 2):
-    CONFIG = configparser.ConfigParser()
-else:
-    CONFIG = configparser.SafeConfigParser()
+CONFIG = configparser.ConfigParser()
 
 LOG = logging.getLogger(__name__)
 
