@@ -19,41 +19,15 @@
 
 """Database setup and migration commands."""
 
-import os
-import threading
-
 from oslo_config import cfg
 from oslo_db import options as db_options
-from stevedore import driver
 
 
-_IMPL = None
-_LOCK = threading.Lock()
-
 db_options.set_defaults(cfg.CONF)
 
 
-def get_backend():
-    global _IMPL
-    if _IMPL is None:
-        with _LOCK:
-            if _IMPL is None:
-                _IMPL = driver.DriverManager(
-                    "glance.database.migration_backend",
-                    cfg.CONF.database.backend).driver
-    return _IMPL
-
-
 # Migration-related constants
 EXPAND_BRANCH = 'expand'
 CONTRACT_BRANCH = 'contract'
-CURRENT_RELEASE = 'train'
+CURRENT_RELEASE = 'xena'
 ALEMBIC_INIT_VERSION = 'liberty'
-LATEST_REVISION = 'rocky_contract02'
-INIT_VERSION = 0
-
-MIGRATE_REPO_PATH = os.path.join(
-    os.path.abspath(os.path.dirname(__file__)),
-    'sqlalchemy',
-    'migrate_repo',
-)
