@@ -22,6 +22,7 @@
 """Defines interface for DB access."""
 
 import datetime
+import itertools
 import threading
 
 from oslo_config import cfg
@@ -746,6 +747,74 @@ def _image_get_disk_usage_by_owner(owner, session, image_id=None):
     return total
 
 
+def _image_get_staging_usage_by_owner(owner, session):
+    # NOTE(danms): We could do this in a single query, but I think it is
+    # easier to understand as two that we concatenate while generating
+    # results.
+
+    # Images in uploading or importing state are consuming staging
+    # space.
+    query = session.query(models.Image)
+    query = query.filter(models.Image.owner == owner)
+    query = query.filter(models.Image.size > 0)
+    query = query.filter(models.Image.status.in_(('uploading',
+                                                  'importing')))
+    importing_images = query.all()
+
+    # Images with non-empty os_glance_importing_to_stores properties
+    # may also be consuming staging space. Filter our deleted images
+    # or importing ones included in the above query.
+    props = session.query(models.ImageProperty).filter(
+        models.ImageProperty.name == 'os_glance_importing_to_stores',
+        models.ImageProperty.value != '').subquery()
+    query = session.query(models.Image)
+    query = query.join(props, props.c.image_id == models.Image.id)
+    query = query.filter(models.Image.owner == owner)
+    query = query.filter(models.Image.size > 0)
+    query = query.filter(~models.Image.status.in_(('uploading',
+                                                   'importing',
+                                                   'killed',
+                                                   'deleted')))
+    copying_images = query.all()
+
+    return sum(i.size for i in itertools.chain(importing_images,
+                                               copying_images))
+
+
+def _image_get_count_by_owner(owner, session):
+    query = session.query(models.Image)
+    query = query.filter(models.Image.owner == owner)
+    query = query.filter(~models.Image.status.in_(['killed', 'deleted']))
+    return query.count()
+
+
+def _image_get_uploading_count_by_owner(owner, session):
+    """Return a count of the images uploading or importing."""
+
+    importing_statuses = ('saving', 'uploading', 'importing')
+
+    # Images in any state indicating uploading, including through image_upload
+    # or importing count for this.
+    query = session.query(models.Image)
+    query = query.filter(models.Image.owner == owner)
+    query = query.filter(models.Image.status.in_(importing_statuses))
+    uploading = query.count()
+
+    # Images that are not in the above list, but are not deleted and
+    # in the process of doing a copy count for this.
+    props = session.query(models.ImageProperty).filter(
+        models.ImageProperty.name == 'os_glance_importing_to_stores',
+        models.ImageProperty.value != '').subquery()
+    query = session.query(models.Image)
+    query = query.join(props, props.c.image_id == models.Image.id)
+    query = query.filter(models.Image.owner == owner)
+    query = query.filter(~models.Image.status.in_(importing_statuses +
+                                                  ('killed', 'deleted')))
+    copying = query.count()
+
+    return uploading + copying
+
+
 def _validate_image(values, mandatory_status=True):
     """
     Validates the incoming data and raises a Invalid exception
@@ -1182,12 +1251,20 @@ def _image_property_update(context, prop_ref, values, session=None):
 
 def image_property_delete(context, prop_ref, image_ref, session=None):
     """
-    Used internally by image_property_create and image_property_update.
+    Used internally by _set_properties_for_image().
     """
     session = session or get_session()
     prop = session.query(models.ImageProperty).filter_by(image_id=image_ref,
                                                          name=prop_ref).one()
-    prop.delete(session=session)
+    try:
+        prop.delete(session=session)
+    except sa_orm.exc.StaleDataError as e:
+        LOG.debug(('StaleDataError while deleting property %(prop)r '
+                   'from image %(image)r likely means we raced during delete: '
+                   '%(err)s'),
+                  {'prop': prop_ref, 'image': image_ref,
+                   'err': str(e)})
+        return
     return prop
 
 
@@ -1554,6 +1631,21 @@ def user_get_storage_usage(context, owner_id, image_id=None, session=None):
     return total_size
 
 
+def user_get_staging_usage(context, owner_id, session=None):
+    session = session or get_session()
+    return _image_get_staging_usage_by_owner(owner_id, session)
+
+
+def user_get_image_count(context, owner_id, session=None):
+    session = session or get_session()
+    return _image_get_count_by_owner(owner_id, session)
+
+
+def user_get_uploading_count(context, owner_id, session=None):
+    session = session or get_session()
+    return _image_get_uploading_count_by_owner(owner_id, session)
+
+
 def _task_info_format(task_info_ref):
     """Format a task info ref for consumption outside of this module"""
     if task_info_ref is None:
