@@ -36,9 +36,6 @@ class TestTasks(functional.FunctionalTest):
         self.cleanup()
         self.api_server.deployment_flavor = 'noauth'
 
-    def _url(self, path):
-        return 'http://127.0.0.1:%d%s' % (self.api_port, path)
-
     def _headers(self, custom_headers=None):
         base_headers = {
             'X-Identity-Status': 'Confirmed',
