@@ -108,7 +108,7 @@ class FakeGateway(object):
         self.policy = policy
         self.repo = repo
 
-    def get_repo(self, context):
+    def get_repo(self, context, authorization_layer=True):
         return self.repo
 
 
@@ -126,6 +126,12 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.controller = glance.api.v2.image_data.ImageDataController()
         self.controller.gateway = FakeGateway(db, store, notifier, policy,
                                               self.image_repo)
+        # FIXME(abhishekk): Everything is fake in this test, so mocked the
+        # image mutable_check, Later we need to fix these tests to use
+        # some realistic data
+        patcher = mock.patch('glance.api.v2.policy.check_is_image_mutable')
+        patcher.start()
+        self.addCleanup(patcher.stop)
 
     def test_download(self):
         request = unit_test_utils.get_fake_request()
@@ -187,6 +193,28 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertEqual('YYYY', image.data)
         self.assertEqual(4, image.size)
 
+    def test_upload_not_allowed_by_policy(self):
+        request = unit_test_utils.get_fake_request()
+        with mock.patch.object(self.controller.policy, 'enforce') as mock_enf:
+            mock_enf.side_effect = webob.exc.HTTPForbidden()
+            exc = self.assertRaises(webob.exc.HTTPNotFound,
+                                    self.controller.upload, request,
+                                    unit_test_utils.UUID1, 'YYYY', 4)
+            self.assertTrue(mock_enf.called)
+        # Make sure we did not leak details of the original Forbidden
+        # error into the NotFound returned to the client.
+        self.assertEqual('The resource could not be found.', str(exc))
+
+        # Now reject the upload_image call, but allow get_image to ensure that
+        # we properly see a Forbidden result.
+        with mock.patch.object(self.controller.policy, 'enforce') as mock_enf:
+            mock_enf.side_effect = [webob.exc.HTTPForbidden(),
+                                    lambda *a: None]
+            exc = self.assertRaises(webob.exc.HTTPForbidden,
+                                    self.controller.upload, request,
+                                    unit_test_utils.UUID1, 'YYYY', 4)
+            self.assertTrue(mock_enf.called)
+
     def test_upload_status(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('abcd')
@@ -217,13 +245,14 @@ class TestImagesController(base.StoreClearingUnitTest):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('abcd', owner='tenant1')
         self.image_repo.result = image
-        mock_enforce.side_effect = exception.Forbidden
+        mock_enforce.side_effect = [exception.Forbidden, lambda *a: None]
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.upload,
                           request, unit_test_utils.UUID2, 'YYYY', 4)
-        expected_target = {'project_id': 'tenant1'}
-        mock_enforce.assert_called_once_with(request.context,
-                                             "upload_image",
-                                             expected_target)
+        expected_call = [
+            mock.call(mock.ANY, 'upload_image', mock.ANY),
+            mock.call(mock.ANY, 'get_image', mock.ANY)
+        ]
+        mock_enforce.has_calls(expected_call)
 
     def test_upload_invalid(self):
         request = unit_test_utils.get_fake_request()
@@ -250,6 +279,27 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertEqual(3, mocked_save.call_count)
         mocked_delete.assert_called_once_with()
 
+    @mock.patch('glance.common.trust_auth.TokenRefresher')
+    def test_upload_with_token_refresh(self, mock_refresher):
+        mock_refresher.return_value = mock.MagicMock()
+        mocked_save = mock.Mock()
+        mocked_save.side_effect = [lambda *a: None,
+                                   exception.NotAuthenticated(),
+                                   lambda *a: None]
+        request = unit_test_utils.get_fake_request()
+        request.environ['keystone.token_info'] = {
+            'token': {
+                'roles': [{'name': 'member'}]
+            }
+        }
+        image = FakeImage('abcd', owner='tenant1')
+        self.image_repo.result = image
+        self.image_repo.save = mocked_save
+        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
+        self.assertEqual('YYYY', image.data)
+        self.assertEqual(4, image.size)
+        self.assertEqual(3, mocked_save.call_count)
+
     def test_upload_non_existent_image_during_save_initiates_deletion(self):
         def fake_save_not_found(self, from_state=None):
             raise exception.ImageNotFound()
@@ -329,7 +379,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.image_repo.result = image
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.upload,
                           request, unit_test_utils.UUID1, 'YYYY', 4)
-        self.assertEqual('killed', self.image_repo.saved_image.status)
+        self.assertEqual('queued', self.image_repo.saved_image.status)
 
     def test_image_size_limit_exceeded(self):
         request = unit_test_utils.get_fake_request()
@@ -459,10 +509,11 @@ class TestImagesController(base.StoreClearingUnitTest):
         request = unit_test_utils.get_fake_request()
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
-        with mock.patch.object(filesystem.Store, 'add'):
+        with mock.patch.object(filesystem.Store, 'add') as mock_add:
+            mock_add.return_value = ('foo://bar', 4, 'ident', {})
             self.controller.stage(request, image_id, 'YYYY', 4)
         self.assertEqual('uploading', image.status)
-        self.assertEqual(0, image.size)
+        self.assertEqual(4, image.size)
 
     def test_image_already_on_staging(self):
         image_id = str(uuid.uuid4())
@@ -470,10 +521,11 @@ class TestImagesController(base.StoreClearingUnitTest):
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
         with mock.patch.object(filesystem.Store, 'add') as mock_store_add:
+            mock_store_add.return_value = ('foo://bar', 4, 'ident', {})
             self.controller.stage(request, image_id, 'YYYY', 4)
             self.assertEqual('uploading', image.status)
             mock_store_add.side_effect = glance_store.Duplicate()
-            self.assertEqual(0, image.size)
+            self.assertEqual(4, image.size)
             self.assertRaises(webob.exc.HTTPConflict, self.controller.stage,
                               request, image_id, 'YYYY', 4)
 
@@ -553,9 +605,11 @@ class TestImagesController(base.StoreClearingUnitTest):
         request = unit_test_utils.get_fake_request()
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
-        self.controller.stage(request, image_id, 'YYYY', 4)
+        with mock.patch.object(filesystem.Store, 'add') as mock_add:
+            mock_add.return_value = ('foo://bar', 4, 'ident', {})
+            self.controller.stage(request, image_id, 'YYYY', 4)
         self.assertEqual('uploading', image.status)
-        self.assertEqual(0, image.size)
+        self.assertEqual(4, image.size)
         # try staging again
         mock_store_add.side_effect = exception.InvalidImageStatusTransition(
             cur_status='uploading', new_status='uploading')
@@ -567,7 +621,8 @@ class TestImagesController(base.StoreClearingUnitTest):
         request = unit_test_utils.get_fake_request()
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
-        with mock.patch.object(filesystem.Store, 'add'):
+        with mock.patch.object(filesystem.Store, 'add') as mock_add:
+            mock_add.return_value = ('foo://bar', 4, 'ident', {})
             self.controller.stage(request, image_id, 'YYYY', 4)
         if expected_url is None:
             self.assertNotIn('os_glance_stage_host', image.extra_properties)
@@ -1014,6 +1069,12 @@ class TestMultiBackendImagesController(base.MultiStoreClearingUnitTest):
         self.controller = glance.api.v2.image_data.ImageDataController()
         self.controller.gateway = FakeGateway(db, store, notifier, policy,
                                               self.image_repo)
+        # FIXME(abhishekk): Everything is fake in this test, so mocked the
+        # image muntable_check, Later we need to fix these tests to use
+        # some realistic data
+        patcher = mock.patch('glance.api.v2.policy.check_is_image_mutable')
+        patcher.start()
+        self.addCleanup(patcher.stop)
 
     def test_upload(self):
         request = unit_test_utils.get_fake_request()
